% File: dn.m
% Date: Jun 17, 2014
% Authors: anca.ciurte@cs.utcluj.ro (Anca Ciurte) and olivier.cuisenaire@gmail.com
% (Olivier Cuisenaire) and xavier.bresson@unil.ch (Xavier Bresson)
% Version: 2.0 beta
% License: GPLv3 - 29 June 2007
%
% Please cite the following paper:
% [1] A. Ciurte, X. Bresson, O. Cuisenaire, N. Houhou, S. Nedevschi, J-Ph. Thiran,
% M. Bach Cuadra, "Semi-Supervised Segmentation of Ultrasound Images based
% on Patch Representation and Continuous Min Cut", PlosOne, 9(7), p. e100972, 2014.
% Short Summary:
% --------------------------------------------------------------------------------------
%
% Copyright (c) 2014, anca.ciurte@cs.utcluj.ro (Anca Ciurte), olivier.cuisenaire@gmail.com
% (Olivier Cuisenaire) and xavier.bresson@unil.ch (Xavier Bresson)
% with Radiology Department, Medical Image Analysis Laboratory (MIAL), CIBM (UNIL
% and CHUV), Signal Processing Laboratory (LTS5 - EPFL),
% and Computer Science Department, Technical University of Cluj-Napoca
% (UTCN). 
% All rights reserved.
%
% This file is part of Fast Patch-based Continuos Min-Cut (fP-CMC)
% segmentation software
%
% fP-CMC is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% fP-CMC is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with fP-CMC. If not, see <http://www.gnu.org/licenses/>.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
% A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
% OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
% LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
% DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
% THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


function wn=dn(w,type);
% function Wn=dn(W,type);
% normalizes a symmetric kernel.  
% D = sum(W).  
% 'ave'   returns D^-1*W
% 'graph' returns D^-1/2*W*D^-1/2
% beltrami

D = sum(w,2);

if type == 'ave'
    D  = 1./D;
    D  = sparse(1:length(D),1:length(D),D);
    wn = D*w;
elseif type == 'gph'
    D  = 1./sqrt(D);
    D  = sparse(1:length(D),1:length(D),D);
    wn = D*(w*D);
end