% File: run.m
% Date: Jun 17, 2014
% Authors: anca.ciurte@cs.utcluj.ro (Anca Ciurte) and olivier.cuisenaire@gmail.com
% (Olivier Cuisenaire) and xavier.bresson@unil.ch (Xavier Bresson)
% Version: 2.0 beta
% License: GPLv3 - 29 June 2007
%
% Please cite the following paper:
% [1] A. Ciurte, X. Bresson, O. Cuisenaire, N. Houhou, S. Nedevschi, J-Ph. Thiran,
% M. Bach Cuadra, "Semi-Supervised Segmentation of Ultrasound Images based
% on Patch Representation and Continuous Min Cut", PlosOne, 9(7), p. e100972, 2014.
% Short Summary:
% --------------------------------------------------------------------------------------
%
% Copyright (c) 2014, anca.ciurte@cs.utcluj.ro (Anca Ciurte), olivier.cuisenaire@gmail.com
% (Olivier Cuisenaire) and xavier.bresson@unil.ch (Xavier Bresson)
% with Radiology Department, Medical Image Analysis Laboratory (MIAL), CIBM (UNIL
% and CHUV), Signal Processing Laboratory (LTS5 - EPFL),
% and Computer Science Department, Technical University of Cluj-Napoca
% (UTCN). 
% All rights reserved.
%
% This file is part of Fast Patch-based Continuos Min-Cut (fP-CMC)
% segmentation software
%
% fP-CMC is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% fP-CMC is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with fP-CMC. If not, see <http://www.gnu.org/licenses/>.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
% A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
% OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
% LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
% DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
% THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


function run(Im0, dirname, filename_w, t, f, sigma2, beta)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 % run.m builds the weighted graph in the form of a sparse adjacency matrix 
 % and saves it in filename_w.mat file

 %INPUT
 % Im0: original image
 % dirname: the path to the original image
 % filename_w: the name of the file where the weihted graph is saved
 % t: size of the searching window
 % f: size of the patch. 
 % sigma2: scaling parameter
 % beta: regularization weigth 
 
  %Example:

  %run(Im0, dirname, filename_w, t, f, sigma2, beta);%
  %OUTPUT: none (the structure of the graph is saved in filename_w.mat)
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%clear, close all
Im0 = double(Im0);
Im0 = (Im0-min(Im0(:)))./(max(Im0(:))-min(Im0(:)));

[Ny,Nx]=size(Im0);

%method=1; %Gaussian additive noise
method=2;  %speckle noise

tic
[GW]=ComputeW0(method,Im0,t,f,sigma2,0.002,0.1,8,struct('distance',3)); 
time_w=toc;

[N1 N2 Nn]=size(GW.weight);
N=N1*N2;

%Construction of the sparse matrix containing the weights between two
%points of the image

tic

is = zeros(1,N*(Nn+1));
js = zeros(1,N*(Nn+1));
ws = ones(1,N*(Nn+1));

ii=0;
for i1 =1:N1
    for i2 =1:N2
        ii=ii+1;
        I = (i2-1)*Ny+i1;
        is(ii) = I;
        js(ii) = I;
        for i3 = 1:Nn
            ii=ii+1;
            is(ii)=I;
            js(ii)=GW.position(i1,i2,i3);
        end
    end
end

SparseW = sparse(is,js,ws); % for efficient pre-allocation

for i1 =1:N1 
    for i2 =1:N2
        i = (i2-1)*Ny+i1;
        SparseW(i,i)= 1;
    end
end

for i1 =1:N1 %t+1:N1-t
    for i2 =1:N2% t+1:N2-tIm0=1+((Im0-min(Im0(:)))./(max(Im0(:))-min(Im0(:)))).*16;
        s = (GW.weight(i1,i2,:)); %suqeeze necessaire???
        j = GW.position(i1,i2,:);
        i = (i2-1)*Ny+i1;%sub2ind(SIZ,i1,i2);
        SparseW(i,j)= s;
    end
end
%fprintf('\tDo it sparse')
time_sparse=toc;

%Symmetrization of the matrix
SparseW=(SparseW+SparseW')./2;

param.sigma2=sigma2;
param.t=t;
param.f=f;
param.beta=beta;

save(filename_w, 'Im0','SparseW', 'GW', 'param', 'time_w', 'time_sparse', 'beta');

end
